/**
 * HorzBarCategoryDialog.java
 */
package UTEStatsViewer;
import java.awt.*;
import java.awt.event.*;

/**
 * Dialog used to change the characteristics of a HorzBarCategory.
 */
final class HorzBarCategoryDialog extends com_ibm_us_bolmar.AWTX.Dialog
      implements ActionListener, MouseListener
{
   // the value entry fields
   private IntTextField redField, greenField, blueField;

   // rectangle in the current color
   private FilledRect currentRect;

   // the OK, Apply, Quit, and Cancel buttons
   private Button okButton = new Button(Message.get(Message.ok));
   private Button applyButton = new Button(Message.get(Message.apply));
   private Button resetButton = new Button(Message.get(Message.reset));
   private Button cancelButton = new Button(Message.get(Message.cancel));
   private Button closeButton = new Button(Message.get(Message.close));
   private Button buttons[] = {okButton, applyButton, resetButton,
         cancelButton, closeButton};

   // Optional Expand and Condense category hierarchy buttons
   private Button expandButton, condenseButton;

   // component whose color is to be changed
   HorzBarCategory target;

   // color the component had before the dialog
   Color colorOrig;

   // width and height of color samples
   private static final int sampleWidth = 21;
   private static final int sampleHeight = 13;

   // the color sample grid and number of cloumns in it
   private Container colorSamplesPane;
   private int colorSamplesRowCnt;
   private int colorSamplesColCnt;

   // Default gap to use between components
   private static final int DEFAULT_HGAP = 0;
   private static final int DEFAULT_VGAP = 5;


   /**
    * Create a customization dialog.
    * @param parentFrame the parent of the dialog
    * @param category the category that is to be customized
    */
   HorzBarCategoryDialog(Frame parentFrame, HorzBarCategory category) {
      super(parentFrame);
      setTitle(Message.get(Message.customize) + " " + category.getName());
      target = category;
      colorOrig = category.getForeground();

      // populate this dialog with its components
      setLayout(new BorderLayout(DEFAULT_HGAP, DEFAULT_VGAP));
      Component expandCondensePane = createExpandCondensePane(category);
      if (expandCondensePane != null) { 
         add(expandCondensePane, BorderLayout.NORTH);
      }
      add(createColorSamples(), BorderLayout.CENTER);
      add(createColorEntry(category.getForeground()), BorderLayout.SOUTH);

      pack();
   }


   /**
    * Tweek the layout done by the standard layout managers.
    */
   public void doLayout() {
      super.doLayout();

      // center the color sample grid in the available space, GridLayout
      // sets each column to the same width, and left justifies the grid;
      // don't resize to avoid its implicit invalidation
      if (colorSamplesPane != null) {
         Rectangle bounds = colorSamplesPane.getBounds();
         bounds.x += (bounds.width % colorSamplesColCnt) / 2;
         bounds.y += (bounds.height % colorSamplesRowCnt) / 2;
         colorSamplesPane.setBounds(bounds);
      }
   }


   /**
    * Return a Component consisting of buttons through which the
    * category can be condensed and expanded.  Returns null if category
    * cannot be expanded or condensed.
    */
   private Component createExpandCondensePane(HorzBarCategory category) {
      Container pane = null;
      boolean hasParent = category.hasParentCategory();
      boolean hasChild  = category.hasChildCategory();
      if (hasParent || hasChild) {
         pane = new com_ibm_us_bolmar.AWTX.Container();
         pane.setLayout(new FlowLayout(FlowLayout.CENTER));

         expandButton = new Button(Message.get(Message.expand));
         expandButton.setEnabled(hasChild);
         pane.add(expandButton);
         expandButton.addActionListener(this);

         condenseButton = new Button(Message.get(Message.condense));
         condenseButton.setEnabled(hasParent);
         pane.add(condenseButton);
         condenseButton.addActionListener(this);
      }
      return pane;
   }


   /**
    * Return Component consisting of small color samples that are
    * clicked on to change the color.
    */
   private Component createColorSamples() {
      // construct container with grid of color sample
      Container pane = new com_ibm_us_bolmar.AWTX.Container();
      final int colorCnt = ColorPalette.getColorCount();
      final int rowCnt = 18;     // number of rows to use
      final int colCnt = (colorCnt+rowCnt-1)/rowCnt;
      colorSamplesColCnt = colCnt;
      colorSamplesRowCnt = rowCnt;

      pane.setLayout(new GridLayout(rowCnt, colCnt));
      for (int i = 0; i < colorCnt; i++) {
         Component comp = new FilledRect(sampleWidth, sampleHeight,
               ColorPalette.getColor(i));
         pane.add(comp);
         comp.addMouseListener(this);
      }

      colorSamplesPane = pane;
      return pane;
   }


   /**
    * Return Component in which the red, green, and blue color values
    * can be changed.
    */
   private Component createColorEntry(Color color) {
      com_ibm_us_bolmar.AWTX.Container ret = new
            com_ibm_us_bolmar.AWTX.Container();
      ret.setLayout(new GridBagLayout());
      GridBagConstraints gbc = new GridBagConstraints();
      gbc.gridx = 0;

      ret.add(currentRect = new FilledRect(55, 34, color), gbc);
      ret.add(createRGBEntryPane(color), gbc);
      ret.add(createButtonPane(), gbc);

      return ret;
   }

   // the range of RGB values
   static final String uByteRangeString = "0 -- 255"; 


   /** Returns the container with the RGB entry fields. */
   private Container createRGBEntryPane(Color color) {
      com_ibm_us_bolmar.AWTX.Container fieldPane = new
            com_ibm_us_bolmar.AWTX.Container();

      // remove empty descent space left by range labels
      fieldPane.setInsets(DEFAULT_VGAP, 0, -3, 0);

      fieldPane.setLayout(new GridBagLayout());
      GridBagConstraints gbc = new GridBagConstraints();
      gbc.gridy = 0;

      // add the red, green, and blue entry fields
      redField = new IntTextField(0, 255, String.valueOf(color.getRed()));
      redField.setForeground(Color.red);
      fieldPane.add(redField, gbc);

      greenField = new IntTextField(0, 255, String.valueOf(color.getGreen()));
      greenField.setForeground(Color.green);
      fieldPane.add(greenField, gbc);

      blueField = new IntTextField(0, 255, String.valueOf(color.getBlue()));
      blueField.setForeground(Color.blue);
      fieldPane.add(blueField, gbc);

      // add the range labels as the second row
      gbc.gridy++;
      for (int i = 0; i < 3; i++) {
         fieldPane.add(new Label(uByteRangeString, Label.CENTER), gbc);
      }

      return fieldPane;
   }


   /** Returns the command button container. */
   private Container createButtonPane() {
      Container buttonPane = new com_ibm_us_bolmar.AWTX.Container();
      buttonPane.setLayout(new FlowLayout());
      for (int i = 0; i < buttons.length; i++) {
         buttonPane.add(buttons[i]);
         buttons[i].addActionListener(this);
      }
      return buttonPane;
   }


   /** Returns whether the color values are valid. */
   private boolean validateColorFields() {
      return false;
   }


   /**
    * Override dispose to allow objects referenced by this one to be
    * garbage collected.
    */
   public void dispose() {
      removeAll();

      redField = greenField = blueField = null;
      currentRect = null;

      okButton = applyButton = resetButton = cancelButton =
           closeButton = null;
      buttons = null;

      target = null;
      colorOrig = null;

      super.dispose();
   }


   /** Implement ActionListener interface. */
   public void actionPerformed(ActionEvent ae) {
      Object source = ae.getSource();
      if (source == closeButton) {
         // do nothing, code to delete dialog after other cases
      } else if (source == cancelButton) {
         // restore original color values, end dialog
         target.setForeground(colorOrig);
      } else if (source == resetButton) {
         // restore original color values
         target.setForeground(colorOrig);
         currentRect.setForeground(colorOrig);
         redField.setText(String.valueOf(colorOrig.getRed()));
         greenField.setText(String.valueOf(colorOrig.getGreen()));
         blueField.setText(String.valueOf(colorOrig.getBlue()));
         currentRect.repaint();
         return;  // stay in dialog
      } else if (source == okButton) {
         if (redField.isValueValid() && greenField.isValueValid() &&
               blueField.isValueValid()) {
            target.setForeground(new Color(redField.getValue(),
                  greenField.getValue(), blueField.getValue()));
         } else {
            getToolkit().beep();
            return;  // stay in dialog
         }
      } else if (source == applyButton) {
         if (redField.isValueValid() && greenField.isValueValid() &&
               blueField.isValueValid()) {
            Color newColor = new Color(redField.getValue(),
                  greenField.getValue(), blueField.getValue());
            target.setForeground(newColor);
            currentRect.setForeground(newColor);
            currentRect.repaint();
         } else {
            getToolkit().beep();
         }
         return;  // stay in dialog
      } else if (source == expandButton) {
         target.expandCategoryHierarchy();
      } else if (source == condenseButton) {
         target.condenseCategoryHierarchy();
      }

      // dialog completed; get rid of it
      dispose();
   }


   /** Implement ItemListener interface. */
   public void itemStateChanged(ItemEvent e) {
      Object source = e.getSource();
   }


   /** Implement MouseListener interface. */
   public void mouseClicked(MouseEvent e) {}

   /**
    * When mouse is pressed, change color of the target to that of
    * the component in which the mouse was pressed.  Stay in dialog
    * which is left by clicking the Quit button.
    */
   public void mousePressed(MouseEvent e) {
      // the AWT sometimes fails to realize when the mouse has moved from
      // a light-weight component to a heavy-weight one and posts mouse
      // events in the heavy-weight component to the last light-weight one
      if (contains(e.getX(), e.getY())) {
         Color newColor = e.getComponent().getForeground();
         currentRect.setForeground(newColor);
         currentRect.repaint();
         target.setForeground(newColor);
         redField.setText(String.valueOf(newColor.getRed()));
         greenField.setText(String.valueOf(newColor.getGreen()));
         blueField.setText(String.valueOf(newColor.getBlue()));
      }
   }

   public void mouseReleased(MouseEvent e) {}
   public void mouseEntered(MouseEvent e) {}
   public void mouseExited(MouseEvent e) {}
}
